# Generated by Django 4.2.16 on 2024-10-29 21:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0227_alter_defaultenterpriseenrollmentintention_content_type_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='defaultenterpriseenrollmentrealization',
            name='realized_enrollment',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='enterprise.enterprisecourseenrollment'),
        ),
    ]



from django.db import connection, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0024_auto_20221031_1855'),
    ]

    db_engine = connection.settings_dict['ENGINE']

    if 'sqlite3' in db_engine:
        operations = [
            migrations.AlterIndexTogether(
                name='cornerstonelearnerdatatransmissionaudit',
                index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
            ),
        ]
    else:
        if 'postgresql' in db_engine:
            operations = [
                migrations.SeparateDatabaseAndState(
                    state_operations=[
                        migrations.AlterIndexTogether(
                            name='cornerstonelearnerdatatransmissionaudit',
                            index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                        ),
                    ],
                    database_operations=[
                        migrations.RunSQL(sql="""
                            CREATE INDEX cornerstone_cldta_85936b55_idx
                            ON cornerstone_cornerstonelearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)
                        """, reverse_sql="""
                            DROP INDEX cornerstone_cldta_85936b55_idx
                        """),
                    ]
                ),
            ]
        else:
            # For MySQL or other non-sqlite and non-postgresql backends
            operations = [
                migrations.SeparateDatabaseAndState(
                    state_operations=[
                        migrations.AlterIndexTogether(
                            name='cornerstonelearnerdatatransmissionaudit',
                            index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                        ),
                    ],
                    database_operations=[
                        migrations.RunSQL(sql="""
                            CREATE INDEX cornerstone_cldta_85936b55_idx
                            ON cornerstone_cornerstonelearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)
                            ALGORITHM=INPLACE LOCK=NONE
                        """, reverse_sql="""
                            DROP INDEX cornerstone_cldta_85936b55_idx
                            ON cornerstone_cornerstonelearnerdatatransmissionaudit
                        """),
                    ],
                ),
            ]



# Generated by Django 3.2.15 on 2023-01-03 14:55

from django.db import connection, migrations


def generate_index_sql(db_engine):
    if 'mysql' in db_engine:
        return """CREATE INDEX sapsf_sldta_85936b55_idx
            ON sap_success_factors_sapsuccessfactorslearnerdatatransmission3ce5 (enterprise_customer_uuid, plugin_configuration_id)
            ALGORITHM = INPLACE LOCK = NONE"""
    elif 'postgresql' in db_engine:
        return """CREATE INDEX sapsf_sldta_85936b55_idx
            ON sap_success_factors_sapsuccessfactorslearnerdatatransmission3ce5 (enterprise_customer_uuid, plugin_configuration_id)"""
    else:
        # Handle other database engines (e.g., SQLite)
        return """CREATE INDEX sapsf_sldta_85936b55_idx
            ON sap_success_factors_sapsuccessfactorslearnerdatatransmission3ce5 (enterprise_customer_uuid, plugin_configuration_id)"""


class Migration(migrations.Migration):
    dependencies = [
        ('sap_success_factors', '0010_move_and_recrete_completed_timestamp'),
    ]

    db_engine = connection.settings_dict['ENGINE']
    if 'sqlite3' in db_engine:
        operations = [
            migrations.AlterIndexTogether(
                name='sapsuccessfactorslearnerdatatransmissionaudit',
                index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
            ),
        ]
    else:
        operations = [
            migrations.SeparateDatabaseAndState(
                state_operations=[
                    migrations.AlterIndexTogether(
                        name='sapsuccessfactorslearnerdatatransmissionaudit',
                        index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                    ),
                ],
                database_operations=[
                    migrations.RunSQL(sql=generate_index_sql(db_engine), reverse_sql="""
                                                                                     DROP INDEX sapsf_sldta_85936b55_idx
                                                                                         ON sap_success_factors_sapsuccessfactorslearnerdatatransmission3ce5
                                                                                     """),
                ]
            ),
        ]



# Generated by Django 4.2.13 on 2024-06-24 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0215_remove_enterprisecustomer_career_engagement_network_message_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='errored_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The last time the membership action was in an error state. Null means the membership is not errored.', null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='errored_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The last time the membership action was in an error state. Null means the membership is not errored.', null=True),
        ),
        migrations.AlterField(
            model_name='enterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending'), ('internal_api_error', 'Internal API error'), ('email_error', 'Email error')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending'), ('internal_api_error', 'Internal API error'), ('email_error', 'Email error')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
    ]



"""
Views for the Enterprise Customer API Credentials.
"""
from edx_rbac.decorators import permission_required
from oauth2_provider.generators import generate_client_id, generate_client_secret
from oauth2_provider.models import get_application_model
from rest_framework import permissions, status
from rest_framework.response import Response

from enterprise.api.utils import assign_feature_roles, has_api_credentials_enabled, set_application_name_from_user_id
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.logging import getEnterpriseLogger

LOGGER = getEnterpriseLogger(__name__)

# Application Model: https://github.com/jazzband/django-oauth-toolkit/blob/master/oauth2_provider/models.py
Application = get_application_model()


class APICredEnabledPermission(permissions.BasePermission):
    """
    Permission that checks to see if the request user matches the user indicated in the request body.
    """

    def has_permission(self, request, view):
        return has_api_credentials_enabled(request.parser_context.get('kwargs', {}).get('enterprise_uuid'))


class APICredentialsViewSet(EnterpriseReadWriteModelViewSet):
    """
    API views for the ``enterprise-customer-api-credentials`` API endpoint.
    """

    # Verifies the requesting user has the appropriate API permissions
    permission_classes = (permissions.IsAuthenticated, APICredEnabledPermission,)

    def get_queryset(self):
        return Application.objects.filter(user=self.request.user)  # only get current user's record

    def get_serializer_class(self):
        return serializers.EnterpriseCustomerApiCredentialSerializer

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, *args, **kwargs: kwargs.get('enterprise_uuid')
    )
    def create(self, request, *args, **kwargs):
        """
        Creates and returns a new enterprise API credential application record.

        Method: POST

        URL:
            /enterprise/api/v1/enterprise-customer-api-credentials/{enterprise_uuid}

        Returns:
            201 if a new API application credentials was created. If an application already exists for the user, throw
            a 409.
        """
        # Verifies the requesting user is connected to an enterprise that has API credentialing bool set to True
        user = request.user
        # Fetches the application for the user
        # If an application already exists for the user, throw a 409.
        queryset = self.get_queryset().first()
        if queryset:
            return Response({'detail': 'Application exists.'}, status=status.HTTP_409_CONFLICT)

        # Adds the appropriate enterprise related feature roles if they do not already have them
        assign_feature_roles(user)

        application = Application.objects.create(
            name=set_application_name_from_user_id(request.user.id),
            user=request.user,
            authorization_grant_type="client-credentials",
            client_type="confidential",
            client_id=generate_client_id(),
            client_secret=generate_client_secret()
        )
        application.save()

        serializer = self.get_serializer(application)
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, *args, **kwargs: kwargs.get('enterprise_uuid')
    )
    def destroy(self, request, *args, **kwargs):
        """
        Removes the enterprise API application credentials for the requesting user.

        Method: DELETE

        URL: /enterprise/api/v1/enterprise-customer-api-credentials/{enterprise_uuid}
        """
        application = Application.objects.filter(user=request.user).first()
        if not application:
            return Response(
                {'detail': 'Application does not exist for requesting user.'},
                status=status.HTTP_400_BAD_REQUEST
            )
        application.delete()
        return Response(status=status.HTTP_200_OK)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, *args, **kwargs: kwargs.get('enterprise_uuid')
    )
    def retrieve(self, request, *args, **kwargs):
        """
        Returns the enterprise API application credentials details for the requesting user.

        Method: GET

        URL: /enterprise/api/v1/enterprise-customer-api-credentials/{enterprise_uuid}
        """
        user_application = Application.objects.get(user=request.user)
        serializer = self.get_serializer(instance=user_application)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, *args, **kwargs: kwargs.get('enterprise_uuid')
    )
    def update(self, request, *args, **kwargs):
        """
        Updates the enterprise API application credentials details for the requesting user.

        Method: PUT

        URL: /enterprise/api/v1/enterprise-customer-api-credentials/{enterprise_uuid}
        """
        # Verifies the requesting user is connected to an enterprise that has API credentialing bool set to True
        user = request.user
        queryset = self.get_queryset().first()
        if not queryset:
            return Response({'detail': 'Could not find the Application.'}, status=status.HTTP_404_NOT_FOUND)

        instance = Application.objects.get(user=user)
        expected_fields = {'name', 'client_type', 'redirect_uris', 'authorization_grant_type'}
        # Throw a 400 if any field to update is not a part of the Application model.
        for field in request.data.keys():
            if field not in expected_fields:
                return Response({'detail': f"Invalid field for update: {field}"}, status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer(instance, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)


class APICredentialsRegenerateViewSet(APICredentialsViewSet):
    """
    API views for the ``enterprise-customer-api-credentials`` API endpoint.
    """

    # Verifies the requesting user has the appropriate API permissions
    permission_classes = (permissions.IsAuthenticated, APICredEnabledPermission,)

    def get_queryset(self):
        return Application.objects.filter(user=self.request.user)  # only get current user's record

    def get_serializer_class(self):
        return serializers.EnterpriseCustomerApiCredentialRegeneratePatchSerializer

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, *args, **kwargs: kwargs.get('enterprise_uuid')
    )
    def update(self, request, *args, **kwargs):
        """
        Regenerates the API application credentials (client ID and secret) for the requesting user. Throws a 404 if the
        user does not yet have any credentials.

        Method: PUT

        URL: /enterprise/api/v1/enterprise-customer-api-credentials/{enterprise_uuid}/regenerate_credentials
        """
        # Fetches the application for the user
        # Throws a 404 if Application record not found
        application = self.get_queryset().first()
        if not application:
            return Response({'detail': 'Could not find the Application.'}, status=status.HTTP_404_NOT_FOUND)

        if 'redirect_uris' not in request.data:
            return Response({'detail': 'Could not update.'}, status=status.HTTP_400_BAD_REQUEST)

        redirect_uris = request.data.get('redirect_uris')
        application.redirect_uris = redirect_uris
        # Calls generate_client_secret and generate_client_id for the user
        application.client_id = generate_client_id()
        application.client_secret = generate_client_secret()
        application.save()

        serializer = self.get_serializer(application)
        return Response(serializer.data, status=status.HTTP_200_OK)



# Generated by Django 4.2.19 on 2025-03-19 22:12
import logging

from django.db import migrations
from django.core.management import call_command


logger = logging.getLogger(__name__)


def run_backfill_command(apps, schema_editor):
    """Runs the backfill management command automatically."""
    try:
        logger.info('Running backfill_ecu_table_user_foreign_key...')
        call_command('backfill_ecu_table_user_foreign_key')
        logger.info('Backfill completed successfully.')
    except Exception as e:  # pylint: disable=broad-except
        logger.warning(f'Warning: The command "backfill_ecu_table_user_foreign_key" failed: {e}')


class Migration(migrations.Migration):
    dependencies = [
        ('enterprise', '0231_alter_enterprisecustomeruser_user_fk_and_more'),
    ]

    operations = [
        migrations.RunPython(run_backfill_command, reverse_code=migrations.RunPython.noop)
    ]
